<?php
/**
 * CodeIgniter library
 *
 * PHP version 5
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Koller
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id$
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object'.EXT;

/**
 * Used to save an ical event
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Ical_event extends Value_object
{
    protected $CI;
    protected $add_or_id;
    
    protected $date;
    protected $from;
    protected $to;
    protected $created;
    protected $organizer;
    protected $location;
    protected $comment;
    protected $category;
    protected $status;
    protected $multistaff;
    protected $participants;

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Ical event ID has to be "add" or an ID';
        }
        
        if ($this->status!='TENTATIVE' && $this->status!='CANCELLED' && $this->status!='CONFIRMED') {
            return 'Unknown Ical status';
        }

        return false;
    }

    /**
     * Init from values
     *
     * @param   mixed   $add_or_id      The item ID or 'add' for adding
     * @param   object  $date           Date
     * @param   object  $from           From time
     * @param   object  $to             To time
     * @param   object  $created        Created date
     * @param   object  $notify         Notify date
     * @param   boolean $is_notify      Notify enabled?
     * @param   integer $organizer      Organizer ID
     * @param   string  $location       Location
     * @param   string  $comment        Comment
     * @param   integer $category       Category ID
     * @param   string  $status         Status string ($this->status!='TENTATIVE' && $this->status!='CANCELLED' && $this->status!='CONFIRMED')
     * @param   array   $multistaff     Multistaff array
     * @param   array   $participants   Participants array
     * @return  object
     */
    public static function init_from_values(
        $add_or_id,
        UsDate $date,
        UsTime $from,
        UsTime $to,
        UsDate $created,
        UsDateTime $notify,
        $is_notify,
        $organizer,
        $location,
        $comment,
        $category,
        $status,
        array $multistaff,
        array $participants
    ) {
        $obj = new Ical_event();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $add_or_id;

        $obj->date = $date;
        $obj->from = $from;
        $obj->to = $to;
        $obj->created = $created;
        $obj->notify_date = $notify;
        $obj->is_notify = $is_notify;
        $obj->organizer = $organizer;
        $obj->location = $location;
        $obj->comment = $comment;
        $obj->category = $category;
        $obj->status = $status;
        $obj->multistaff = $multistaff;
        $obj->participants = $participants;

        return $obj;
    }


    /**
     * Returns the names of all variables of this object that should be serialized
     *
     * @return  array
     */
    public function __sleep()
    {
        return array('add_or_id', 'date', 'from', 'to', 'created', 'notify_date', 'is_notify', 'organizer', 'location',
            'comment', 'category', 'status', 'multistaff', 'participants');
    }

    /**
     * Reconstruct any resources
     *
     * @return  void
     */
    public function __wakeup()
    {
        $this->CI = CI_Controller::get_instance();
    }

    /**
     * Gain access to all properties
     *
     * @param   string  $name   Name of the property
     * @return  mixed
     */
    public function get($name)
    {
        if (property_exists($this, $name)) {
            return $this->{$name};
        } else {
            throw new Exception("Property not defined");
        }
    }

    /**
     * Get a uniqe id
     *
     * @return  mixed
     */
    public function get_uniqe_id()
    {
        $id = $this->get('add_or_id');
        if ($id=='add') {
            return us_uniqid('', true);
        } else {
            return $id;
        }
    }

} // END class Ical_event
