<?php
/**
 * CodeIgniter library
 *
 * PHP version 5
 *
 * @category CodeIgniter
 * @license /_licenses/untermStrich/agb_normal.pdf
 * @author Christian Wengg
 * @copyright 2009 untermStrich
 * @package us_libraries
 * @version $Id: staff_values.php 22369 2021-04-14 14:29:15Z christian $
*/

if (!defined('BASEPATH')) exit('No direct script access allowed');

require_once APPPATH.'us_libraries/Value_object_addi'.EXT;

/**
 * Used to save a staff entry
 *
 * @property    My_Controller           $CI                     The main CI object
 */
class Staff_values extends Value_object_addi
{

    private   $_staffadmin_changeable_security_fields;
    private   $_staffadmin_changeable_security_detail_fields;

    protected $staff_name;
    protected $username;
    protected $street;
    protected $zip_code;
    protected $town;
    protected $phone_1;
    protected $phone_2;
    protected $fax;
    protected $email;
    protected $gender;
    protected $insurance_no;
    protected $birthday;
    protected $start_date;
    protected $end_date;
    protected $company_id;
    protected $staff_location_id;
    protected $remark;
    protected $photo_file;
    protected $bank_code;
    protected $bank_name;
    protected $account_no;
    protected $abbreviation;
    protected $direct_dial;
    protected $annex;
    protected $holiday_rate_id;
    protected $exclude_from_turnover;
    protected $respect_project_manager;
    protected $change_days_past;
    protected $days_warning_no_workinput;
    protected $days_warning_no_mail_allocation;
    protected $is_project_manager;
    protected $project_manager_id;

    /**
     * Init the staff values from $_POST
     *
     * @return  object  An Staff_values object with all values out of the post array
     */
    public static function init_from_post()
    {
        $obj = new Staff_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $obj->CI->input->post('staffadmin_edit');
        $obj->staff_name = $obj->CI->input->post('staffadmin_staff_name');
        $obj->username = $obj->CI->input->post('staffadmin_username');
        $obj->street = $obj->CI->input->post('staffadmin_street');
        $obj->zip_code = $obj->CI->input->post('staffadmin_zip_town__zip');
        $obj->town = $obj->CI->input->post('staffadmin_zip_town__town');
        $obj->phone_1 = $obj->CI->input->post('staffadmin_phone_1');
        $obj->phone_2 = $obj->CI->input->post('staffadmin_phone_2');
        $obj->fax = $obj->CI->input->post('staffadmin_fax');
        $obj->email = $obj->CI->input->post('staffadmin_email');
        $obj->gender = $obj->CI->input->post('staffadmin_gender');
        $obj->insurance_no = $obj->CI->input->post('staffadmin_insurance_no');
        $obj->birthday = new UsDate($obj->CI->input->post('staffadmin_birthday'), null, true, null);
        $obj->start_date = new UsDate($obj->CI->input->post('staffadmin_start_date'), null, true, null);
        $obj->end_date = new UsDate($obj->CI->input->post('staffadmin_end_date'), null, true, null);
        $obj->company_id = $obj->CI->input->post('staffadmin_company');
        $obj->staff_location_id = $obj->CI->input->post('staffadmin_staff_location');
        $obj->remark = $obj->CI->input->post('staffadmin_remark');
        $obj->photo_file = $obj->CI->input->post('staffadmin_photo_file');
        $obj->bank_code = $obj->CI->input->post('staffadmin_bank_code');
        $obj->bank_name = $obj->CI->input->post('staffadmin_bank_name');
        $obj->account_no = $obj->CI->input->post('staffadmin_account_no');
        $obj->abbreviation = $obj->CI->input->post('staffadmin_abbreviation');
        $obj->direct_dial = $obj->CI->input->post('staffadmin_direct_dial');
        $obj->annex = $obj->CI->input->post('staffadmin_annex');
        $obj->holiday_rate_id = $obj->CI->input->post('staffadmin_holiday_rate');
        $obj->exclude_from_turnover = $obj->CI->input->post('staffadmin_exclude_from_turnover') ? true : false;
        $obj->respect_project_manager = $obj->CI->input->post('staffadmin_respect_project_manager') ? true : false;
        $obj->change_days_past = $obj->CI->input->post('staffadmin_change_days_past');
        $obj->days_warning_no_workinput = $obj->CI->input->post('staffadmin_days_warning_no_workinput');
        $obj->days_warning_no_mail_allocation = $obj->CI->input->post('staffadmin_days_warning_no_mail_allocation');
        $obj->is_project_manager = $obj->CI->input->post('staffadmin_is_project_manager') ? true : false;
        $obj->project_manager_id = $obj->CI->input->post('staffadmin_project_manager');
        $obj->password = $obj->CI->input->post('staffadmin_password');
        $obj->confirmed_password = $obj->CI->input->post('staffadmin_confirmed_password');
        
        $multitenancy = $obj->CI->multitenancy->get_info();
        if ($multitenancy->can_set_superuser()) {
            $obj->is_superuser = $obj->CI->input->post('staffadmin_is_superuser') ? true : false;
        }        

        $obj->addi_vars = $obj->CI->staff->get_addi_vars_post();

        $obj->CI->load->model('Staff_model', 'staff');
        $obj->_staffadmin_changeable_security_fields = $obj->CI->staff->get_staffadmin_changeable_security_fields();
        $obj->_staffadmin_changeable_security_detail_fields = $obj->CI->staff->get_staffadmin_changeable_security_detail_fields();

        foreach ($obj->_staffadmin_changeable_security_fields as $field) {
            $field = 'security_'.$field;
            $obj->$field = $obj->CI->input->post('staffadmin_'.$field);
        }

        return $obj;
    }

    /**
     * Init the staff values from a given StdClass object
     *
     * @param   object  $values StdClass object with staff values to initialize a valid Staff_values object out of it
     *
     * @return  object  An Staff_values object with all values out of the given StdClass object
     */
    public static function init_from_values(StdClass $values)
    {
        $obj = new Staff_values();
        $obj->CI = CI_Controller::get_instance();
        $obj->add_or_id = $values->id;
        $obj->staff_name = $values->staff_name;
        $obj->username = $values->username;
        $obj->street = $values->street;
        $obj->zip_code = $values->zip_code;
        $obj->town = $values->town;
        $obj->phone_1 = $values->phone_1;
        $obj->phone_2 = $values->phone_2;
        $obj->fax = $values->fax;
        $obj->email = $values->email;
        $obj->gender = $values->gender;
        $obj->insurance_no = $values->insurance_no;
        $obj->birthday = $values->birthday;
        $obj->start_date = $values->start_date;
        $obj->end_date =$values->end_date;
        $obj->company_id = $values->company_id;
        $obj->staff_location_id = isset($values->staff_location_id) ? $values->staff_location_id : 0;
        $obj->remark = $values->remark;
        $obj->photo_file = $values->photo_file;
        $obj->bank_code = $values->bank_code;
        $obj->bank_name = $values->bank_name;
        $obj->account_no = $values->account_no;
        $obj->abbreviation = $values->abbreviation;
        $obj->direct_dial = $values->direct_dial;
        $obj->annex = $values->annex;
        $obj->holiday_rate_id = $values->holiday_rate_id;
        $obj->exclude_from_turnover = (boolean) $values->exclude_from_turnover;
        $obj->respect_project_manager = (boolean) $values->respect_project_manager;
        $obj->change_days_past = float_to_usnumber($values->change_days_past);
        $obj->days_warning_no_workinput = float_to_usnumber($values->days_warning_no_workinput);
        $obj->days_warning_no_mail_allocation = float_to_usnumber($values->days_warning_no_mail_allocation);
        $obj->is_project_manager = (boolean) $values->is_project_manager;
        $obj->project_manager_id = $values->project_manager_id;
        $obj->password = $values->password;
        $obj->confirmed_password = $values->password;
        
        $multitenancy = $obj->CI->multitenancy->get_info();
        if ($multitenancy->can_set_superuser() && isset($values->is_superuser)) {
            $obj->is_superuser = (boolean) $values->is_superuser;
        }         

        $a_addi = array();
        foreach ($values->addi_fields as $addi) {
            $addi_field = $addi->field;
            $a_addi[$addi_field] = $values->$addi_field;
        }
        $obj->addi_vars = $a_addi;

        $obj->CI->load->model('Staff_model', 'staff');
        $obj->_staffadmin_changeable_security_fields = $obj->CI->staff->get_staffadmin_changeable_security_fields();
        $obj->_staffadmin_changeable_security_detail_fields = $obj->CI->staff->get_staffadmin_changeable_security_detail_fields();

        foreach ($obj->_staffadmin_changeable_security_fields as $field) {
            $field = 'security_'.$field;
            $obj->$field = $values->$field;
        }
        return $obj;
    }

    /**
     * Validate and ensure all values are appropriate
     *
     * @return  mixed   false on success, else the error message
     */
    public function validate()
    {
        if ($this->add_or_id!=='add' && !is_numeric($this->add_or_id)) {
            return 'Staff ID has to be "add" or an ID';
        }
        
        if ($this->add_or_id==='add' && (!$this->password || $this->password == '' || ($this->password != $this->confirmed_password))) {
            return 'Incorrect password';
        }        

        return false;
    }

    /**
     * Get staff to save
     *
     * @return  array
     */
    public function get_staff_to_save()
    {
        $this->CI->load->model('System_settings_model', 'system_settings');
        $this->CI->load->model('Staff_model', 'staff');

        $only_same_mgmt = $this->CI->system_settings->getStaffadmin_only_same_mgmt();
        $staff_right = $this->CI->login->get_right(Login_model::R_STAFFADMIN);
        
        $data = array(
            'staff_name'                        =>  $this->staff_name,
            'username'                          =>  $this->username,
            'street'                            =>  $this->street,
            'zip_code'                          =>  $this->zip_code,
            'town'                              =>  $this->town,
            'phone_1'                           =>  $this->phone_1,
            'phone_2'                           =>  $this->phone_2,
            'fax'                               =>  $this->fax,
            'email'                             =>  $this->email,
            'gender'                            =>  $this->gender,
            'insurance_no'                      =>  $this->insurance_no,
            'birthday'                          =>  $this->birthday->mysql(),
            'start_date'                        =>  $this->start_date->mysql(),
            'end_date'                          =>  $this->end_date->mysql(),
            'company_id'                        =>  intval($this->company_id),
            'staff_location_id'                 =>  intval($this->staff_location_id),
            'remark'                            =>  $this->remark,
            'bank_code'                         =>  $this->bank_code,
            'bank_name'                         =>  $this->bank_name,
            'account_no'                        =>  $this->account_no,
            'abbreviation'                      =>  $this->abbreviation,
            'direct_dial'                       =>  $this->direct_dial,
            'annex'                             =>  $this->annex,
            'holiday_rate_id'                   =>  intval($this->holiday_rate_id),
            'exclude_from_turnover'             =>  $this->exclude_from_turnover,
            'respect_project_manager'           =>  $this->respect_project_manager,
            'change_days_past'                  =>  intval($this->change_days_past),
            'days_warning_no_workinput'         =>  intval($this->days_warning_no_workinput),
            'days_warning_no_mail_allocation'   =>  intval($this->days_warning_no_mail_allocation),
            'is_project_manager'                =>  $this->is_project_manager,
            'project_manager_id'                =>  intval($this->project_manager_id),
        );
        
        if (isset($this->is_superuser)) {
            $data['is_superuser'] = $this->is_superuser;
        }        
        
        if ($this->photo_file!==false && !empty($this->photo_file)) {
            if($this->photo_file=='#DELETEME#') {
                $data['photo_file'] = null;
            } else {
                $data['photo_file'] = $this->photo_file;
            }
        }
        
        if ($this->add_or_id==='add') {
            $data['password'] = $this->password;
        }
        
        foreach ($this->_staffadmin_changeable_security_fields as $field) {
            if (!in_array($field, $this->_staffadmin_changeable_security_detail_fields)) {
                continue; //Nothing to do
            }
            $field_main = 'security_'.$field;
            $field_detail = 'secd_'.$field;
            $split_data = explode('__', $this->$field_main);
            if (count($split_data)>=2) {
                $this->$field_main = $split_data[0];
                $this->$field_detail = $split_data[1];
            } else {
                $this->$field_detail = 0;
            }
        }
        
        if ($only_same_mgmt) {
            if ($this->add_or_id==='add') {
                foreach ($this->_staffadmin_changeable_security_fields as $field) {
                    $field_main = 'security_'.$field;
                    if (intval($this->$field_main)<$staff_right) {
                        return false;
                    }
                }
            } else {
                $staff_object = $this->CI->staff->get_item(intval($this->add_or_id));
                if ($staff_object===null) {
                    return false;
                }
                
                foreach ($this->_staffadmin_changeable_security_fields as $field) {
                    $field_main = 'security_'.$field;
                    if (intval($staff_object->$field_main)<$staff_right) {
                        return $data;
                    }
                }
            }
        }
        
        foreach ($this->_staffadmin_changeable_security_fields as $field) {
            $field_main = 'security_'.$field;
            $data[$field_main] = intval($this->$field_main);
            
            if (in_array($field, $this->_staffadmin_changeable_security_detail_fields)) {
                $field_detail = 'secd_'.$field;
                $data[$field_detail] = intval($this->$field_detail);
            }
        }

        return $data;
    }

} // END class Staff_values
